#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "symtab.h"
#include "darray.h"
#include "get_time.h"

int main()
{
    darray_t d;
    symtab_t ht, cbt;
    symtab_init_cbt(cbt);
    symtab_init_hashtab(ht);
    darray_init(d);
    int i;
    double t0, t1;

    void add_key(char *key)
    {
	darray_append(d, key);
    }
    void randomize(void)
    {
	for (i=d->count-1; i>1; i--) {
	    darray_swap(d, i, random() % i);
	}
    }

    FILE *fp = stdin;
    for (;;) {
	char buf[80];
	char *status = fgets(buf, 80, fp);
	if (feof(fp)) break;
	int i = strlen(buf) - 1;
	if (buf[i] == '\n') buf[i] = '\0';
	add_key(strdup(buf));
    }

    randomize();

    if (0) {
	// print contents of darray
	void print(void *data) {
	    printf(" %s", (char *) data);
	}
	printf("%d", d->count);
	darray_forall(d, print);
	printf("\n");
    }

    t0 = get_time();
    for (i=0; i<d->count; i++) {
	symtab_put(cbt, d->item[i], d->item[i]);
    }
    t1 = get_time();
    printf("Crit-bit tree insertion time: %lf\n", t1 - t0);
    t0 = get_time();
    for (i=0; i<d->count; i++) {
	symtab_put(ht, d->item[i], d->item[i]);
    }
    t1 = get_time();
    printf("Hash table insertion time: %lf\n", t1 - t0);
 
    randomize();

    t0 = get_time();
    for (i=0; i<d->count; i++) {
	if (strcmp(d->item[i], symtab_at(cbt, d->item[i]))) {
	    fprintf(stderr, "BUG!\n");
	    exit(1);
	}
    }
    t1 = get_time();
    printf("Crit-bit tree lookup time: %lf\n", t1 - t0);
    t0 = get_time();
    for (i=0; i<d->count; i++) {
	if (strcmp(d->item[i], symtab_at(ht, d->item[i]))) {
	    fprintf(stderr, "BUG!\n");
	    exit(1);
	}
    }
    t1 = get_time();
    printf("Hash table lookup time: %lf\n", t1 - t0);
 
    return 0;
}
