#include <stdlib.h>
#include "list.h"

void list_init(list_ptr l)
{
    l->sentinel->next = l->sentinel;
    l->sentinel->prev = l->sentinel;
    l->count = 0;
}

static void list_node_insert(list_node_ptr node, void *data)
{
    list_node_ptr p = malloc(sizeof(list_node_t));
    p->data = data;
    p->next = node;
    node->prev->next = p;
    p->prev = node->prev;
    node->prev = p;
}

static void list_node_append(list_node_ptr node, void *data)
{
    list_node_ptr p = malloc(sizeof(list_node_t));
    p->data = data;
    p->prev = node;
    node->next->prev = p;
    p->next = node->next;
    node->next = p;
}

void list_insert(list_ptr l, void *data)
{
    l->count++;
    list_node_append(l->sentinel, data);
}

void list_append(list_ptr l, void *data)
{
    l->count++;
    list_node_insert(l->sentinel, data);
}

void list_forall(list_ptr l, void (*f)(void *))
{
    list_node_ptr p = l->sentinel->next;
    while(p != l->sentinel) {
	f(p->data);
	p = p->next;
    }
}

void list_iterator_init(list_iterator_ptr it, list_ptr l)
{
    it->list = l;
    it->node = l->sentinel->next;
}

void list_iterator_insert(list_iterator_ptr it, void *data)
{
    it->list->count++;
    list_node_insert(it->node, data);
}

void list_iterator_append(list_iterator_ptr it, void *data)
{
    it->list->count++;
    list_node_append(it->node, data);
}

void list_iterator_next(list_iterator_ptr it)
{
    it->node = it->node->next;
}

void list_iterator_prev(list_iterator_ptr it)
{
    it->node = it->node->prev;
}

void list_iterator_first(list_iterator_ptr it)
{
    it->node = it->list->sentinel->next;
}

void list_iterator_last(list_iterator_ptr it)
{
    it->node = it->list->sentinel->prev;
}

int list_iterator_is_end(list_iterator_ptr it)
{
    return it->node == it->list->sentinel;
}
