#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "symtab.h"
#include "cbt.h"
#include "hashtab.h"

// wrappers for crit-bit trees
static symtab_cl_t symtab_cbt;

static inline cbt_ptr cbt_cast(symtab_ptr ht)
{
    if (ht->cl != symtab_cbt) {
	fprintf(stderr, "bug: not a crit-bit tree\n");
    }
    return ht->data;
}

static void *cbt_at_(symtab_ptr ht, char *key)
{
    cbt_ptr cbt = cbt_cast(ht);
    return cbt_at(cbt, key);
}

static void cbt_put_(symtab_ptr ht, void *data, char *key)
{
    cbt_ptr cbt = cbt_cast(ht);
    cbt_put(cbt, data, key);
}

static void cbt_print_(symtab_ptr ht)
{
    cbt_ptr cbt = cbt_cast(ht);
    cbt_print(cbt);
}

static void cbt_init_(symtab_ptr ht)
{
    ht->data = cbt_alloc();
    cbt_ptr cbt = cbt_cast(ht);
    cbt_init(cbt);
}

void symtab_init_cbt(symtab_ptr ht)
{
    static int initialized = 0;
    if (!initialized) {
	symtab_cbt->at = cbt_at_;
	symtab_cbt->put = cbt_put_;
	symtab_cbt->print = cbt_print_;
	initialized = 1;
    }
    ht->cl = symtab_cbt;
    cbt_init_(ht);
}

// wrappers for hash tables
static symtab_cl_t symtab_hashtab;

static inline hashtab_ptr hashtab_cast(symtab_ptr ht)
{
    if (ht->cl != symtab_hashtab) {
	fprintf(stderr, "bug: not a hash table\n");
    }
    return ht->data;
}

static void *hashtab_at_(symtab_ptr ht, char *key)
{
    hashtab_ptr hashtab = hashtab_cast(ht);
    return hashtab_at(hashtab, key);
}

static void hashtab_put_(symtab_ptr ht, void *data, char *key)
{
    hashtab_ptr hashtab = hashtab_cast(ht);
    hashtab_put(hashtab, data, key);
}

static void hashtab_print_(symtab_ptr ht)
{
    hashtab_ptr hashtab = hashtab_cast(ht);
    hashtab_print(hashtab);
}

static void hashtab_init_(symtab_ptr ht)
{
    ht->data = hashtab_alloc();
    hashtab_ptr hashtab = hashtab_cast(ht);
    hashtab_init(hashtab);
}

void symtab_init_hashtab(symtab_ptr ht)
{
    static int initialized = 0;
    if (!initialized) {
	symtab_hashtab->at = hashtab_at_;
	symtab_hashtab->put = hashtab_put_;
	symtab_hashtab->print = hashtab_print_;
	initialized = 1;
    }
    ht->cl = symtab_hashtab;
    hashtab_init_(ht);
}
